@@ -10,7 +10,7 @@ import yaml
 
 from .config import merge_cfgs
 from .images import import_images
-from .utils import load_cfg_file, oc, wait_for_ready_threaded
+from .utils import cancel_builds, load_cfg_file, oc, wait_for_ready_threaded
 from .secrets import import_secrets, SecretImporter
 from .templates import get_templates_in_dir
 
@@ -89,7 +89,7 @@ def deploy_components(
         bcs = template.get_processed_names_for_restype("bc")
         for name in bcs:
             log.info("Re-triggering builds for '%s'", name)
-            oc("cancel-build", "bc/{}".format(name), state="pending,new,running")
+            cancel_builds(name)
             oc("start-build", "bc/{}".format(name))
 
     # Wait on all resources that have been marked as 'resources to wait for'
@@ -162,6 +162,12 @@ def _get_custom_methods(service_set, service_set_dir, root_custom_dir):
         module = _load_module(
             os.path.join(root_custom_dir, f"deploy_{service_set}.py"), service_set
         )
+
+    if not module:
+        module = _load_module(
+            os.path.join(root_custom_dir, f"deploy.py"), service_set
+        )
+
     if not module:
         log.exception("Error loading custom deploy script, using default deploy methods")
         return DEFAULT_DEPLOY_METHODS
@@ -266,7 +272,7 @@ class DeployRunner(object):
         self.service_sets_selected = service_sets_selected
         self.resources_scale_factor = resources_scale_factor
         self._deployed_service_sets = []
-        self.specific_components = specific_components
+        self.specific_components = specific_components or []
         self._pick_service_sets = [comp.split("/")[0] for comp in self.specific_components]
         self.label = label
         self.skip = skip
